function [ results,labels ] = calc_microstate_features( data,maps,srate,thresh )
% CALC_MICROSTATE_FEATURES(data,maps,srate) fits input maps onto supplied
% data and calculates several features of interest:
%   -- Average lifespan of each microstate
%   -- Average frequency of each microstate
%   -- Coverage of each  microstate
%   -- GFP peaks/appearance of each microstate and globally
%   -- Average correlation of each microstate
%   -- Global average lifespan
%   -- Global average frequency
%
%   Inputs:
%       data          =     Input data is a cell array of size (time points
%                           x channels x epochs).
%       maps          =     Maps to be fit onto the
%                           supplied data. Has shape (maps x channels).
%       srate         =     Sampling rate in Hz. 
%       thresh        =     [OPTIONAL] If a threshold is set, only fits
%                           above the threshold are counted. The "unfit"
%                           maps are counted as a separate microstate.
%
%   Outputs:
%       results       =     Table display of results 
%       labels        =     Cell array of size (epochs x 1). Each cell
%                           supplies the microstate labels for each point,
%                           and is of size (points x 1).

%% Initialize relevant variables
microstates = size(maps,1) + 1; % The last microstate is the 'unfit' microstate.
peaks = cell(size(data,3),1);
locs = cell(size(data,3),1);
labels = cell(size(data,3),1);
assignments = zeros(microstates,1);
unique_appearances = zeros(microstates,1);
lifespan = zeros(microstates,1);
correlations = zeros(microstates,1);

for i = 1:size(data,3) % for each epoch,
    
    %% Extract GFP Peaks from input data
    [peaks{i},locs{i}] = gfp_peaks(data(:,:,i));
    
    %% Fit microstate maps onto gfp_peaks
    % Get unsigned correlation matrix
    correlation = abs(corr(peaks{i}',maps'));
    
    % Look for best assignments
    [max_corr,labels{i}] = max(correlation,[],2);
    
    % Check threshold
    labels{i}(max_corr<thresh) = microstates; % the last microstate is the 'unfit' microstate

    % Check for errors
    if length(labels{i}) ~= size(peaks{i},1)
        size(correlation)
        peaks{i}
        labels{i}
        error('Number of labels and data points do not match.');
    end
    
    %% Calculate values
    
    % Our strategy is to count the unique instances and sum all of the
    % parameters of interest. From there, we can calculate mean values
    % after all epochs have been scanned through.
    
    for k=2:(length(labels{i})-1) % Ignore the first and last labels of each epoch

        assignments(labels{i}(k)) = assignments(labels{i}(k)) + 1; % add one to original map assignment
        lifespan(labels{i}(k)) = lifespan(labels{i}(k)) + (locs{i}(k+1) + locs{i}(k)) / 2 - (locs{i}(k-1) + locs{i}(k)) / 2; % Add to the total life of this microstate
        correlations(labels{i}(k)) = correlations(labels{i}(k)) + max_corr(k); % Add to the fit of this microstate
        
        if labels{i}(k) ~= labels{i}(k-1) % If this is a unique instance,
            unique_appearances(labels{i}(k)) = unique_appearances(labels{i}(k)) + 1; % Add one to the unique values
        end
        
    end
    
        
end

%% Caclulate final values

% initialize a results cell array, will convert to table later
results = {size(data,3)};
headers = {'Epochs'}; % a set of headers for the columns in 'results'

%   Average lifespan
for m=1:microstates-1 % for the real microstates,
    headers = cat(1,headers,{['Avg_Life_',num2str(m),'_ms']});
    results=cat(1,results,lifespan(m)/unique_appearances(m)/srate*1000);
end
headers = cat(1,headers,{'Avg_Life_Unfit_ms'});
results=cat(1,results,lifespan(microstates)/unique_appearances(microstates)/srate*1000);

%   Frequency
for m=1:microstates-1
    headers = cat(1,headers,{['Freq_',num2str(m),'_Hz']});
    results=cat(1,results,unique_appearances(m)/sum(lifespan/srate));
end
headers = cat(1,headers,{'Freq_Unfit_Hz'});
results=cat(1,results,unique_appearances(microstates)/sum(lifespan/srate));

%   Coverage
for m=1:microstates-1
    headers = cat(1,headers,{['Coverage_',num2str(m)]});
    results=cat(1,results,lifespan(m)/sum(lifespan)*100);
end
headers = cat(1,headers,{'Coverage_Unfit'});
results=cat(1,results,lifespan(microstates)/sum(lifespan)*100);

%   GFP Peaks/Appearance
for m=1:microstates-1
    headers = cat(1,headers,{['GFP_peaks_per_appear_',num2str(m),'_pks']});
    results=cat(1,results,assignments(m)/unique_appearances(m));
end
headers = cat(1,headers,{'GFP_peaks_per_appear_Unfit_pks'});
results=cat(1,results,assignments(microstates)/unique_appearances(microstates));

%   Correlations
for m=1:microstates-1
    headers = cat(1,headers,{['Avg_Corr_',num2str(m)]});
    results=cat(1,results,correlations(m)/assignments(m));
end
headers = cat(1,headers,{'Avg_Corr_Unfit'});
results=cat(1,results,correlations(microstates)/assignments(microstates));

%   Global lifespan (not including unfit)
headers = cat(1,headers,{'Global_Avg_Life_ms'});
results=cat(1,results,sum(lifespan(1:end-1)/srate*1000)/sum(unique_appearances(1:end-1)));

%   Global frequency (not includeing unfit)
headers = cat(1,headers,{'Global_Freq_Hz'});
results=cat(1,results,sum(unique_appearances(1:end-1))/sum(lifespan/srate));

%   Global GFP Peaks/appear (not including unfit)
headers = cat(1,headers,{'Global_GFP_peaks_per_appear'});
results=cat(1,results,sum(assignments(1:end-1))/sum(unique_appearances(1:end-1)));

%   Avg GFP Peaks/sec
headers = cat(1,headers,{'Avg_GFP_peaks_per_sec'});
results=cat(1,results,sum(assignments)/sum(lifespan/srate));

%% Put everything in a table

results = cell2table(results');
results.Properties.VariableNames = headers';
