function [ gev, cv ] = eval_clusters( maps, data )
% EVAL_CLUSTERS(maps, data) inputs a set of microstate maps and data. The
% function calculates the global explained variance of the maps onto the
% data, and calculates a modified cross-validation criteria.
%
%   Inputs:
%       maps     =     Microstate template maps. Has shape (maps x
%                      channels).
%       data     =     Input data onto which maps are fit. Has shape (time
%                      points x channels).
%
%   Outputs:
%       gev      =     Global explained variance.
%       cv       =     Modified cross-validation criteria.

%% Fit the maps onto the data

% Normalize columnwise
maps = normc(maps);

% Get the unsigned correlation matrix between data and maps
correlation = abs(corr(data',maps'));

% Find the best fits
[~,assignments] = max(correlation,[],2);

 %% GFP-normalize the maps for subsequent calculations
 maps = bsxfun(@rdivide,maps,std(maps,[],2));
 
 %% Get GFPs of data
 gfp = std(data,[],2);
 
 %% Calculate global explained variance (gev) for data using maps
 explained_variance = zeros(1,size(data,1));
 for i = 1:size(data,1)
     correlation = dot(data(i,:),maps(assignments(i),:))/(sqrt(dot(data(i,:),data(i,:)))*sqrt(dot(maps(assignments(i),:),maps(assignments(i),:))));
     explained_variance(i) = correlation^2;
 end
 
 gev = sum(gfp .* explained_variance')^2/sum(gfp)^2;
 
 %% Calculate CV
 microstates = size(maps,1); % number of microstates
 nch = size(data,2); % number of channels
 resid_var = 0; % predictive residual variance
 
 for i = 1:size(data,1)
     resid_var = resid_var + (1-abs(corr(data(i,:)',maps(assignments(i),:)')))^2;
 end
 resid_var = resid_var / (length(assignments) * (nch - 1));
 
 % CV is modified version of predictive residual variance
 cv = resid_var * ((nch - 1)/(nch - 1 - microstates))^2;

end

