function [ best_maps, best_assn, gev ] = find_microstates_kmeans( data, mstates, reps, iters, plotOn, chanlocs )
% FIND_MICROSTATES(data,range) takes input data (points x channels) and
% uses a modified k-means algorithm to identify a certain number of
% microstate maps within range ([min max]).
%
%   Inputs:
%       data     =     Input data (samples x channels x epochs).
%       mstates  =     Number of microstates to be found.
%       reps     =     Repetitions of the clustering algorithm; usually 3-5
%                      is sufficient.
%       iters    =     Iterations allowed within each repetition.
%       plotOn   =     [OPTIONAL] Default is 0. If set to 1, results and
%                      cluster evaluations will be plotted.
%       chanlocs =     [OPTIONAL] If plotOn = 1, need chanlocs to plot the
%                      maps.
%
%   Outputs:
%       best_maps =    Matrix of shape (map x channels).
%       best_assn =    Matrix of shape (label x 1).
%       gev      =     Global explained variance. Has shape (1 x
%                      microstates)

tic;

%% Concatenate the epochs of 'data'
o_data = [];
for i = 1:size(data,3)
    o_data = cat(1,o_data,data(:,:,i));
end
data = o_data;

disp(['Looking for ',num2str(mstates),' microstates...']);

%% Repeat entire process over several repetitions to reduce noise.
k = 1;
while k <= reps
    k = k + 1; % Using a while loop so we can re-do certain reps if they fail.
    skip_rep = 0; % Do not skip this repetition
    
    %% Pick mstates random maps to initialize clustering
    maps = data(randi(size(data,1),mstates,1),:);
    
    %% Normalize columnwise
    maps = normc(maps);
    
    %% Initialize two assignment matrices
    old_assignments = zeros(size(data,1),1);
    assignments = old_assignments + 1;
    
    %% Initialize best fit
    best_fit = 0;
    iterations = 0;
    
    %% Loop until old_assignments = new_assignments
    while ~isequal(old_assignments,assignments) && iterations < iters
        
        %% Count iterations
        iterations = iterations + 1;
        
        %% Update the assignment matrix from the last loop
        old_assignments = assignments;
        
        %% Get unsigned correlation matrix for EEG
        correlation = abs(corr(data',maps'));
        
        %% Look for best assignments
        [~,assignments] = max(correlation,[],2);
        
        %% Compute first principal component of each cluster
        for i = 1:mstates
            coeff = pca(data(assignments==i,:));
            
            % Occasionally, a cluster can collapse (no points assigned
            % to a certain cluster). In this case, we'll continue the
            % while loop and repeat this repetition.
            
            if size(coeff,2)==0 % This cluster has collapsed!
                skip_rep = 1;
                break;
            end
            
            maps(i,:) = coeff(:,1);
        end
        
        %% If a cluster has collapsed, repeat repetition.
        if skip_rep == 1
            continue;
        end
        
        %% Normalize columnwise
        maps = normc(maps);
        
        %% Get unsigned correlation matrix for EEG
        correlation = abs(corr(data',maps'));
        
        %% Look for best assignments
        [loading,assignments] = max(correlation,[],2);
        
        %% What is the quality of the fit?
        total_fit = sum(loading);
        
        %% If this is the best fit, save these microstates
        if total_fit > best_fit
            best_maps = double(maps);
            best_assn = assignments;
        end
        
    end
    
    %% Display results of this repetition.
    if skip_rep == 1
        warning('Cluster has collapsed. Repeating this repetition. The ratio of data points to clusters is too low.');
        k = k - 1; % repeat this repetition
    elseif iterations == iters
        disp(['>> Repetition ',num2str(k-1),': clusters failed to converge after ',num2str(iters),' iterations.']);
    else
        disp(['>> Repetition ',num2str(k-1),': optimized in ',num2str(iterations),' iterations.']);
    end
    
end

% TESTING: Implemented 5/8/16. Convert best maps to z-score to optimize display of best microstate maps.
best_maps = zscore(best_maps,0,2);

%% Calculate global explained variance (gev) for model using best_maps

[gev,~] = eval_clusters(best_maps,data);

%% Check to see if plotOn is 1 or 0
if nargin<5
    plotOn = 0;
elseif nargin==5
    warning('To plot results, you must input channel locations.');
    plotOn = 0;
end

if plotOn == 1,
    
    %% Plot the microstates
    figure;
    for j = 1:mstates
        subplot(1,mstates,j);
        topoplot(best_maps(j,:),chanlocs);
    end
    
end

toc;

end