function [ best_maps ] = find_microstates_taahc( data, range, plotOn, chanlocs )
% FIND_MICROSTATES(data,range) takes input data (points x channels) and
% uses a modified heirarchical clustering algorithm to identify a certain number of
% microstate maps.
%
%   Inputs:
%       data     =     Input data (samples x channels x epochs).
%       range    =     Range of microstates to be found [min max].
%       plotOn   =     [OPTIONAL] Default is 0. If set to 1, results and
%                      cluster evaluations will be plotted.
%       chanlocs =     [OPTIONAL] If plotOn = 1, need chanlocs to plot the
%                      maps.
%
%   Outputs:
%       best_maps =    Cell array of size (1 x range(2)). Each cell is a
%                      matrix of shape (map x channels).

tic;



%% Concatenate the epochs of 'data'
o_data = [];
for i = 1:size(data,3)
    o_data = cat(1,o_data,data(:,:,i));
end
data = o_data;

disp(['Looking for ',num2str(range(1)),'-',num2str(range(2)),' microstates...']);

%% Initialize waitbar
h = waitbar(0,'Calculating...');

%% Initialize cluster arrays
cluster_members = cell(size(data,1),1);
cluster_templates = zeros(size(data));

%% Calculate summated GFP for subsequent calculation of GEV
total_gfp = sum(std(data,[],2));

%% All original maps are template maps to start
for i = 1:size(data,1)
    cluster_members{i} = data(i,:);
    cluster_templates(i,:) = data(i,:); % the i'th row of cluster_templates is the template for the cluster in cluster_members{i}
end

%% Loop until number of maps > min range
while size(cluster_members,1) > range(1),
    
    %% Update the waitbar
    waitbar((size(data,1) - size(cluster_templates,1)) / (size(data,1) - range(1)));
    
    %% Find the 'worst' cluster = one with lowest GEV
    quality = zeros(size(cluster_members,1),1);
    for i = 1:size(cluster_members,1) % for each cluster
        for j = 1:size(cluster_members{i},1) % for each member within cluster
            quality(i) = quality(i) + (std(cluster_members{i}(j,:),[],2)*abs(corr(cluster_members{i}(j,:)',cluster_templates(i,:)')))^2 / total_gfp^2;
        end
    end
    [~,delete_me] = min(quality);
    
    %% Get the 'worst' cluster members
    maps_to_be_reassigned = cluster_members{delete_me};
    
    %% Eliminate the 'worst' cluster
    cluster_members(delete_me) = [];
    cluster_templates(delete_me,:) = [];
    
    %% Reassign the atomized maps
    % Calculate correlation between atomized maps and cluster templates
    correlation = abs(corr(maps_to_be_reassigned',cluster_templates'));
    % Identify highest correlation
    [~,reassignments] = max(correlation,[],2);
    % Distribute atomized maps to new clusters
    for i = 1:size(reassignments,1)
        cluster_members{reassignments(i)} = cat(1,cluster_members{reassignments(i)},maps_to_be_reassigned(i,:));
    end
    
    %% Recalculate the template maps of the reassigned clusters
    for i = 1:size(reassignments,1)
        coeff = pca(cluster_members{reassignments(i)});
        cluster_templates(reassignments(i),:) = coeff(:,1);
    end
    
    %% Store maps if # of maps is within range
    if size(cluster_templates,1) <= range(2)
        % TESTING: Implemented 5/8/16. Convert best maps to z-score to optimize display of best microstate maps.
        best_maps{size(cluster_templates,1)} = zscore(normc(cluster_templates),0,2);
    end
    
end


%% Close waitbar
close(h);

if nargin == 4 && plotOn == 1,
    %% Plot the microstates
    figure;
    for i = range(1):range(2) % for each number of requested clusters
        disp(['Plotting ',num2str(i),' clusters.']);
        for j = 1:i % for subplots 1 to i
            
            subplot(range(2)-range(1)+1,range(2),range(2)*(i-range(1)+1)-range(2)+j);
            topoplot(best_maps{i}(j,:),chanlocs);
            
        end
        
    end
    
end

toc;

end